/*
 * gui.c 
 */

/*
** Copyright (C) 1999 Dirk-Jan C. Binnema <djcb@dds.nl>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include "telegnome.xpm"
#include "gui.h"
#include "main.h"

static Gui gui;

/*
 * return the app gui, with startpage or NULL
 */
GtkWidget *
new_gui (gchar* startpage) 
{
	GtkWidget *app, *toolbar, *pixmap, *vbox, *statusbar;

	/* initial zoom-factor=1 */
	gui.zoom_factor=1;

	
	/* the app */
	app= gnome_app_new (PACKAGE, _("TeleGNOME: Teletekst for GNOME"));
	gtk_widget_realize(GTK_WIDGET(app));
	vbox= gtk_vbox_new (0,0);

	toolbar= new_toolbar();
	gnome_app_set_toolbar(GNOME_APP(app), GTK_TOOLBAR(toolbar));
	
	/* the picture, initialize with logo xpm */
	pixmap= new_pixmap (app->window, telegnome_xpm);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), pixmap);

	/* the statusbar */
	statusbar= gnome_appbar_new(FALSE,TRUE,TRUE);
	gnome_app_set_statusbar(GNOME_APP(app), statusbar);

	/* all the contents */
	gnome_app_set_contents(GNOME_APP(app), vbox);

	/* save some pointers for reference later */
	gui.statusbar= statusbar;
	gui.app= app;
	gui.pixmap= GTK_BIN(pixmap)->child;

	return app;
}


/*
 * create a new toolbar 
 */
GtkWidget *
new_toolbar ()
{
	GtkWidget *icon, *zoombutton, *zoomlabel, *toolbar, *entry;
	toolbar= gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
	gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);

	/* add the entry */
	entry= new_entry();
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("Page"),  _("Current Page Number"),
				_("Current Page Number"), entry, NULL, NULL);

	/* add some menu options */

	icon = gnome_stock_pixmap_widget(toolbar, GNOME_STOCK_MENU_JUMP_TO);
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("Go to Page"),  _("Go To Page"),
				_("Go To Page"), icon, cb_goto_page, NULL);

	icon = gnome_stock_pixmap_widget(toolbar, GNOME_STOCK_MENU_BACK);
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("Previous"),  _("Get Previous Page"),
				_("Get Previous Page"), icon, cb_prev_page, NULL);

	icon = gnome_stock_pixmap_widget(toolbar, GNOME_STOCK_MENU_FORWARD);
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("Next"), _("Get Next Page"),
				_("Get Next Page"), icon, cb_next_page, NULL);

	/* toggle button for zoom/unzoom */
	zoomlabel=gtk_label_new("400%");
	zoombutton= gtk_toggle_button_new(); 
	gtk_widget_show(GTK_WIDGET(zoomlabel)); /* somehow, we need this */
	gtk_container_add (GTK_CONTAINER(zoombutton), zoomlabel);
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("Zoom"),  _("Zoom in/Zoom out"),
				_("Zoom in/Zoom out"), zoombutton, cb_zoom, NULL);
	/* save zoombutton for later reference */
	gui.zoombutton= zoombutton;
	gui.zoomlabel= zoomlabel;
	gtk_signal_connect(GTK_OBJECT(zoombutton), "toggled", 
			   GTK_SIGNAL_FUNC(cb_zoom), NULL);



	icon = gnome_stock_pixmap_widget(toolbar, GNOME_STOCK_MENU_ABOUT);
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("About"), _("About TeleGNOME"),
				_("About TeleGNOME"), icon, cb_about, NULL);

	
	/* the exit button */
	icon = gnome_stock_pixmap_widget(toolbar, GNOME_STOCK_MENU_QUIT);
	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), 
				_("Quit"), _("Exit TeleGNOME"), 
				_("Exit TeleGNOME"), icon, cb_quit, NULL);

	return toolbar;
}


/* 
 * create a new entry 
 */
GtkWidget * 
new_entry ()
{
	GtkWidget *entry=NULL;
	entry=gtk_entry_new_with_max_length(TG_PAGE_SIZE + 1 + TG_SUBPAGE_SIZE);
	
	/* hack */
	gtk_widget_set_usize(GTK_WIDGET(entry),
			     (7 * (TG_PAGE_SIZE + 1 + TG_SUBPAGE_SIZE)),20);
	
	/*gtk_entry_set_text(GTK_ENTRY(entry), startpage);*/
	      
	gtk_signal_connect(GTK_OBJECT(entry), "activate", 
			   GTK_SIGNAL_FUNC(cb_goto_page),NULL);

	/* save entry for later ref */
	gui.entry= entry;

	return entry;
}

/*
 * build a new pixmap widget in an GtkEventBox holder
 */
GtkWidget*
new_pixmap (GdkWindow* window, gchar** xpm_data)
{
	static GtkTargetEntry targets[]= {
	  {"text/plain"                , 0, TARGET_TXT},
	  {"STRING"                    , 0, TARGET_TXT},
	  {"image/gif"                 , 0, TARGET_GIF},
	  {"application/x-rootwin-drop", 0, TARGET_ROOTWIN}
	};
	static gint n_targets= sizeof(targets)/sizeof(targets[0]);

	GdkPixmap *gdkpixmap=NULL;
	GdkBitmap *gdkbitmap=NULL;
	GtkWidget *pixmap=NULL;
	GtkWidget *eventbox= gtk_event_box_new();  /* we need an eventbox for DnD */

	gdkpixmap= gdk_pixmap_create_from_xpm_d (window, &gdkbitmap, NULL, xpm_data);	
	pixmap= gtk_pixmap_new(gdkpixmap, gdkbitmap);

	/* clean up */
	gdk_imlib_free_bitmap(gdkbitmap);
	gdk_imlib_free_pixmap(gdkpixmap);

	/* wrap the pic inside an eventbox & and add DnD stuff */
	gtk_container_add(GTK_CONTAINER(eventbox), pixmap);
	gtk_drag_source_set (eventbox, GDK_BUTTON1_MASK,
		       targets, n_targets, 
		       GDK_ACTION_COPY);
	
	gtk_signal_connect(GTK_OBJECT(eventbox), "drag_data_get",
			   GTK_SIGNAL_FUNC(cb_drag), NULL);

	/* save for later ref */
	gui.eventbox=eventbox;
       
	
	return eventbox;
}

/*
 * print a string in the statusbar
 */
int
print_in_statusbar(gchar *buf)  /*FIXME: buffersize*/
{
	gnome_appbar_set_status(GNOME_APPBAR(gui.statusbar), buf);
	gtk_widget_show(GTK_WIDGET(gui.statusbar));
	return 0;
}

/*
 * update the entry box with the current values of page & subpage
 */
int
update_entry(gint page_nr, gint subpage_nr)
{
	gchar full_num[10];

	if (subpage_nr > 0)
		sprintf(full_num,"%d/%d", page_nr, subpage_nr);
	else
		sprintf(full_num,"%d", page_nr);
	gtk_entry_set_text(GTK_ENTRY(gui.entry), full_num);
	
	return 0;
}

/*
 * update the pixmap widget. 
 * if (redraw) we're changing the zoom factor and we have to redraw the whole app 
 */
int
update_pixmap (gchar *filename, gboolean redraw)
{
	gint width, height;
	GdkImlibImage *image;
	GdkPixmap *gdkpixmap;
	GdkBitmap *gdkbitmap;

	/* hide the app */
	if (redraw)
		gtk_widget_hide(GTK_WIDGET(gui.app));

	if (NULL==filename) {
		print_in_statusbar(_("Error getting page"));
		return -1;
	}
    
	image= gdk_imlib_load_image (filename);
	if (!image) {
		print_in_statusbar (_("Error opening image file"));
		return -1;
	}
	width= image->rgb_width; 
	height= image->rgb_height;	

	/* render image using the specified zoom factor */
	gdk_imlib_render(image,
			 gui.zoom_factor * width, 
			 gui.zoom_factor * height);

	gdkpixmap= gdk_imlib_move_image (image);
	gdkbitmap= gdk_imlib_move_mask  (image);

	gtk_pixmap_set(GTK_PIXMAP(gui.pixmap), gdkpixmap, gdkbitmap);
	gtk_widget_show(GTK_WIDGET(gui.pixmap));
	
	/* re-show our holder widget */
	if (redraw) 
		gtk_widget_show_all (GTK_WIDGET(gui.app));

	update_entry(telegnome.page_nr, telegnome.subpage_nr);
	print_in_statusbar ("");

	/* clean up */
	gdk_imlib_free_bitmap(gdkbitmap);
	gdk_imlib_free_pixmap(gdkpixmap);
	gdk_imlib_kill_image(image);

	return 0;
}

/*
 * try to get the page, and do something smart if it fails
 * if redraw == TRUE, redraws whole app
 * (needed when zoom_factor has changed)
 */
gint
get_the_page(gboolean redraw)
{
	gint retval;
	gchar *picfile;

	/* save these the restore the, if necessary */
	gint old_page= telegnome.page_nr;
	gint old_subpage= telegnome.subpage_nr;
	
	retval= get_the_image(&picfile);
	
	if (TG_OK == retval) {
		update_pixmap(picfile, redraw);
		return 0;
	} else {
		switch(retval) {
		case TG_ERR_TOOSMALL:	/* we got an error from the webpage */
			/* maybe we forgot the subpage nr, or used it when we shouldn't */
			telegnome.subpage_nr= (0 == telegnome.subpage_nr)?1:0;
			if (TG_OK != get_the_image(&picfile)) { 
				telegnome.page_nr= old_page;  /* restore */
				telegnome.subpage_nr= old_subpage;
				print_in_statusbar (_("Connection to Teletekst webserver failed"));
				return -1;
		        } else {
				update_pixmap(picfile, redraw);
				return 0;
			}		
		case TG_ERR_GHTTP:
			print_in_statusbar (_("Error making HTTP connection"));
			return -1;
		case TG_ERR_NOCONNECTION:
			print_in_statusbar (_("No connection"));
			return -1;
		case TG_ERR_FILE:
			print_in_statusbar (_("Error using temporary file"));
			return -1;
		case TG_ERR_CHMOD:
			print_in_statusbar (_("Error setting permissions on temporary file"));
			return -1;
		case TG_ERR_HTTPQUERY:
			print_in_statusbar (_("Internal error in HTTP query code"));
			return -1;
		default: 
			print_in_statusbar (_("Internal error in HTTP code"));
			g_assert_not_reached();
			return -1;
		}
	}
}


/* 
 * callbacks 
 */
void
cb_quit (GtkWidget* widget, gpointer data)
{
	g_print (_("Thank you for using TeleGNOME....\n"));
	gtk_main_quit();
}

void
cb_about (GtkWidget* widget, gpointer data)
{
	GtkWidget *about;
	const gchar *authors[]= { N_("Dirk-Jan C. Binnema <djcb@dds.nl>"), NULL };
	about= gnome_about_new (PACKAGE, VERSION,
				_("(c) 1999 Dirk-Jan C. Binnema"),
				authors,
				_("Teletekst for GNOME\nReleased under the terms of the GPL"), 
				NULL);
	gtk_widget_show (GTK_WIDGET(about));
}

void
cb_next_page (GtkWidget* widget, gpointer data)
{	
	telegnome.subpage_nr=0;
	telegnome.page_nr++;
	update_entry(telegnome.page_nr, telegnome.subpage_nr);
	get_the_page(FALSE); /* dont redraw */ 

}

void
cb_prev_page (GtkWidget* widget, gpointer data)
{	
	telegnome.subpage_nr=0;
	telegnome.page_nr--;
	update_entry(telegnome.page_nr, telegnome.subpage_nr);
	get_the_page(FALSE);
}

void
cb_goto_page (GtkWidget *widget, gpointer data)
{	
	      
	if ( -1 == get_page_entry (gtk_entry_get_text(GTK_ENTRY(gui.entry)))) {
		print_in_statusbar(_("Error in page entry"));
		return;
	}
	get_the_page(FALSE);
}


/*
 * handler for zoom button
 */
void 
cb_zoom(GtkWidget *widget, gpointer data)
{
	if (GTK_IS_TOGGLE_BUTTON(widget)) { 
		if (GTK_TOGGLE_BUTTON(widget)->active) {
			gtk_label_set_text(GTK_LABEL(gui.zoomlabel),"100%");
			gui.zoom_factor=2;
		} else {
			gtk_label_set_text(GTK_LABEL(gui.zoomlabel),"400%");
			gui.zoom_factor=1;
		}		
	}
	/* now, get the page with the new zoom settings */
	get_the_page(TRUE);
}

void
cb_drag (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
	 guint info, guint32 time)
{
	gchar *entry;

	switch (info) {

	case TARGET_GIF:
		break;
		
	case TARGET_FILE:
		break;

	case TARGET_ROOTWIN:
		break;

	case TARGET_TXT:

	default:	
		entry=gtk_entry_get_text(GTK_ENTRY(gui.entry));
		gtk_selection_data_set(selection_data, 
				       selection_data->target, 8,
				       entry, strlen(entry));
		
		break;
	}
}



































