/* http.c */
/*
** Copyright (C) 1999 Dirk-Jan C. Binnema <djcb@dds.nl>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/
  
#include <ghttp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "http.h"
#include "main.h"

/*
 * get the pagenumber from the entrybox 
 */
int
get_page_entry (gchar *page_entry)
{	
	guint page_nr;
	guint subpage_nr=0;
	errno=0;
	
	if (strlen (page_entry) > 3) {  
		/* split in page and subpage */
		gchar *subpage_entry= &page_entry[TG_PAGE_SIZE + 1];
		page_entry[TG_PAGE_SIZE]='\0';
		subpage_nr= strtol(subpage_entry,NULL,10);
	}
   
	page_nr= strtol(page_entry,NULL,10);
	if (0 != errno)
		return -1;
   
	telegnome.page_nr=    page_nr;
	telegnome.subpage_nr= subpage_nr;

	return 0;
}

/*
 * get the image asychronously  - by spawning a thread
 * return error if thread takes too long.
 */
static gint get_image_completed;
static gint get_image_retval;

gint
get_the_image_async (gchar **filename) 
{
	pthread_t thread;
	get_image_completed= 0;
	if ( 0 != ( pthread_create( &thread,  NULL,  (void*)get_the_image, (void*)filename))) {
		return -1;
	}
     
	sleep (TG_TIMEOUT);
	
	if (get_image_completed) 
		return get_image_retval;
	else {
		pthread_kill ( thread, -9 );
		return TG_ERR_HTTPQUERY;
	} 
}

/*
 * get the image from a remote site
 * if all's ok, return name in buffer (buffer shouldn't be freed) 
 */
gint
get_the_image (gchar **filename) 
{
	gint output;
	gint bytes_written;
	gchar http_query[100];
	gint retval=0;
	ghttp_request *request= NULL;
	
	if ( -1 == get_http_query(http_query, telegnome.page_nr, telegnome.subpage_nr))	
	return get_image_retval= TG_ERR_HTTPQUERY; 
	*filename=tmpnam(NULL); 
		
	/* get the image from remote server */
	request= ghttp_request_new();
	if (-1 == ghttp_set_uri (request, http_query)) retval= TG_ERR_GHTTP;
	if (-1 == ghttp_set_sync (request, ghttp_sync)) retval= TG_ERR_GHTTP;
	if (-1 == ghttp_set_type (request, ghttp_type_get)) retval= TG_ERR_GHTTP;
	if (-1 == ghttp_prepare(request)) retval= TG_ERR_GHTTP;
	if (ghttp_done != ghttp_process(request)) retval= TG_ERR_GHTTP;
	
	/* write to file */
	if (-1 == (output=open(*filename, O_WRONLY | O_CREAT | O_SYNC | O_TRUNC))) 
		retval = TG_ERR_FILE;
	bytes_written= write(output, ghttp_get_body(request), ghttp_get_body_len(request));
	fsync(output); close(output);
	if ( bytes_written < PAGE_MINSIZE ) {
		get_image_completed = 1;
		return get_image_retval= TG_ERR_TOOSMALL;
	}
	
	/* set read permissions */
	if (0 != chmod (*filename, S_IRUSR)) retval= TG_ERR_CHMOD; 
	else {
		ghttp_request_destroy(request);
		get_image_completed = 1;
		return get_image_retval=TG_OK;
	}
	
	if (!(TG_ERR_FILE == retval)) 
		close (output);
	ghttp_request_destroy(request);     

	get_image_completed = 1;
	return get_image_retval=retval;
}

int
get_http_query (gchar* buffer, gint page_nr, gint subpage_nr)
{
	if ( subpage_nr ) {    /* do we have a subpage? */
		sprintf (buffer, HTTP_REQUEST_SUBPAGE, page_nr, subpage_nr);
	} else {
		sprintf (buffer, HTTP_REQUEST_PAGE, page_nr);
	}
        return 0;
}












